Index: uspace/lib/libc/include/rwlock.h
===================================================================
--- uspace/lib/libc/include/rwlock.h	(revision 9413c0d0defe05c5ae4a86bbc19f4184d15f5da3)
+++ uspace/lib/libc/include/rwlock.h	(revision 9413c0d0defe05c5ae4a86bbc19f4184d15f5da3)
@@ -0,0 +1,56 @@
+/*
+ * Copyright (c) 2008 Jakub Jermar
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * - Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - The name of the author may not be used to endorse or promote products
+ *   derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/** @addtogroup libc
+ * @{
+ */
+/**
+ * @file
+ * @brief	This file contains rwlock API and provides its fake
+ *		implementation based on futexes.
+ */
+
+#ifndef LIBC_RWLOCK_H_
+#define LIBC_RWLOCK_H_
+
+#include <atomic.h>
+#include <sys/types.h>
+#include <futex.h>
+
+typedef atomic_t rwlock_t;
+
+#define rwlock_initialize(rwlock)	futex_initialize((rwlock), 1)
+#define rwlock_reader_lock(rwlock)	futex_down((rwlock))
+#define rwlock_writer_lock(rwlock)	futex_down((rwlock))
+#define rwlock_reader_unlock(rwlock)	futex_up((rwlock))
+#define rwlock_writer_unlock(rwlock)	futex_up((rwlock))
+
+#endif
+
+/** @}
+ */
Index: uspace/srv/vfs/vfs_node.c
===================================================================
--- uspace/srv/vfs/vfs_node.c	(revision f57f8ea58fde79450cfca65fd2c8f15643d3fb1c)
+++ uspace/srv/vfs/vfs_node.c	(revision 9413c0d0defe05c5ae4a86bbc19f4184d15f5da3)
@@ -41,4 +41,5 @@
 #include <atomic.h>
 #include <futex.h>
+#include <rwlock.h>
 #include <libadt/hash_table.h>
 
@@ -147,5 +148,5 @@
 		node->index = triplet->index;
 		link_initialize(&node->nh_link);
-		futex_initialize(&node->contents_futex, 1);
+		rwlock_initialize(&node->contents_rwlock);
 		hash_table_insert(&nodes, key, &node->nh_link);
 	} else {
Index: uspace/srv/vfs/vfs_rdwr.c
===================================================================
--- uspace/srv/vfs/vfs_rdwr.c	(revision f57f8ea58fde79450cfca65fd2c8f15643d3fb1c)
+++ uspace/srv/vfs/vfs_rdwr.c	(revision 9413c0d0defe05c5ae4a86bbc19f4184d15f5da3)
@@ -40,5 +40,5 @@
 #include <async.h>
 #include <errno.h>
-#include <futex.h>
+#include <rwlock.h>
 
 static void vfs_rdwr(ipc_callid_t rid, ipc_call_t *request, bool read)
@@ -86,5 +86,8 @@
 	 * the same time.
 	 */
-	futex_down(&file->node->contents_futex);
+	if (read)
+		rwlock_reader_lock(&file->node->contents_rwlock);
+	else
+		rwlock_writer_lock(&file->node->contents_rwlock);
 
 	int fs_phone = vfs_grab_phone(file->node->fs_handle);	
@@ -118,5 +121,8 @@
 	 * Unlock the VFS node.
 	 */
-	futex_up(&file->node->contents_futex);
+	if (read)
+		rwlock_reader_unlock(&file->node->contents_rwlock);
+	else
+		rwlock_writer_unlock(&file->node->contents_rwlock);
 
 	/*

Index: uspace/srv/vfs/vfs.h
===================================================================
--- uspace/srv/vfs/vfs.h	(revision f57f8ea58fde79450cfca65fd2c8f15643d3fb1c)
+++ uspace/srv/vfs/vfs.h	(revision 9413c0d0defe05c5ae4a86bbc19f4184d15f5da3)
@@ -37,4 +37,5 @@
 #include <libadt/list.h>
 #include <atomic.h>
+#include <rwlock.h>
 #include <sys/types.h>
 #include <bool.h>
@@ -136,6 +137,6 @@
 	link_t nh_link;		/**< Node hash-table link. */
 
-	/** Holding this futex prevents modifications of the node's contents. */
-	atomic_t contents_futex;
+	/** Holding this rwlock prevents modifications of the node's contents. */
+	atomic_t contents_rwlock;
 } vfs_node_t;
 